# Physical Computing with Graphics on Pico Explorer
# Tony Goodhew 25th Feb 2021 (Tested on Vers: 0.0.7)
# Hackspace Mag
import picoexplorer as display
import utime, random, math
width = display.get_width()
height = display.get_height()
display_buffer = bytearray(width * height * 2)
display.init(display_buffer)

def blk():
    display.set_pen(0,0,0)
    display.clear()
    display.update()
    
def title(msg,r,g,b):
    blk()
    display.set_pen(r,g,b)
    display.text(msg, 20, 70, 200, 4)
    display.update()
    utime.sleep(2)
    blk()
    
def show(tt):
    display.update()
    utime.sleep(tt)
    
def horiz(l,t,r):    # left, right, top
    n = r-l+1        # Horizontal line
    for i in range(n):
        display.pixel(l + i, t)

def vert(l,t,b):   # left, top, bottom
    n = b-t+1      # Vertical line
    for i in range(n):
        display.pixel(l, t+i)

def box(l,t,r,b):  # left, top, right, bottom
    horiz(l,t,r)   # Hollow rectangle
    horiz(l,b,r)
    vert(l,t,b)
    vert(r,t,b)

def line(x,y,xx,yy): # (x,y) to (xx,yy)
    if x > xx:
        t = x  # Swap co-ordinates if necessary
        x = xx
        xx = t
        t = y
        y = yy
        yy = t
    if xx-x == 0:  # Avoid div by zero if vertical
        vert(x,min(y,yy),max(y,yy))
    else:          # Draw line one dot at a time L to R
        n=xx-x+1
        grad = float((yy-y)/(xx-x))  # Calculate gradient
        for i in range(n):
            y3 = y + int(grad * i)
            display.pixel(x+i,y3)  # One dot at a time

def align(n, max_chars):
    # Aligns string of n in max_chars
    msg1 = str(n)
    space = max_chars - len(msg1)
    msg2 = ""
    for m in range(space):
        msg2 = msg2 +" "
    msg2 = msg2 + msg1
    return msg2  # String - ready for display

def ring(cx,cy,rr,rim): # Centre, radius, thickness
    display.circle(cx,cy,rr)
    display.set_pen(0,0,0) # background colour
    display.circle(cx,cy,rr-rim)
    
def ring2(cx,cy,r):   # Centre (x,y), radius
    for angle in range(0, 90, 2):  # 0 to 90 degrees in 2s
        y3=int(r*math.sin(math.radians(angle)))
        x3=int(r*math.cos(math.radians(angle)))
        display.pixel(cx-x3,cy+y3)  # 4 quadrants
        display.pixel(cx-x3,cy-y3)
        display.pixel(cx+x3,cy+y3)
        display.pixel(cx+x3,cy-y3)
        
def showgraph(v,rr,gg,bb):   # Bar graph
    display.set_pen(255,0,0)
    display.text("P", 8, 190, 240, 3)
    display.set_pen(0,0,0)        # Blank old bar graph
    display.rectangle(29, 190, 220, 20)
    display.set_pen(int(rr*2.55),int(gg*2.55),int(bb*2.55))    # New  bar graph
    if v == 1:vert(29, 190, 65) # Ensure these show up
    if v == 2:vert(30, 190, 65)
    display.rectangle(29, 190, v, 15)
    display.set_pen(100,190,100)  # Base line zero
    vert(28, 182, 210)             
    display.set_pen(0,0,255)      # percentage
    display.text(str(align(v,4)), 140, 189, 240, 3)
    display.text("%", 210, 190, 20, 3)

# Lines
blk()
title("Lines and circles",0,200,0)
display.set_pen(255,0,0)
for i in range(0,240,10):
    line(0,i,i,239)
    line(239-i,0,0,i) 
    line(239,239-i,i,239)
    line(i,0,239,i)  
#    display.update() # Try uncommented
display.set_pen(0,0,255)
box(0,0,239,239)
display.update()
# Circles
box(0,0,239,239)
for r in range(65):
    display.set_pen(0,r,2*r)
    ring2(119,119,r)
    display.update()
utime.sleep(1)
display.set_pen(200,200,0)
ring(119,119,70,4)
show(2)
for r in range(0,60,10):
    display.set_pen(90,90,90  )
    ring2(119,119,r)
    display.update()
show(2)

# Character Set - Only upper case!
title("Character set",200,200,0)
display.set_pen(0,200,0)
display.text("Character Set", 15, 15, 200, 2)
s = ""
counter = 0
for m in range(32,128,8):
    for n in range(0,8,1):
        p = m + n
        if ((p < 97) or (p>122)): # Not lower case
            s = s + chr(p)
            counter = counter + 1
            if (counter)/16 == int((counter)/16):
                s = s +" "  # 'space' for text wrap
display.set_pen(200,200,0)
display.text(s, 15, 40, 200, 2)
display.set_pen(0,0,200)
display.text("Only upper case", 140, 110, 200, 1)
display.text("Size 3", 78, 130, 200, 3)
display.set_pen(0,200,0)
display.text("Size 4", 60, 156, 200, 4)
display.set_pen(200,0,0)
display.text("Size 6", 34, 190, 200, 6)
show(5)
blk()

# === Sin & Cos graphs ====
title("Plotting graphs",0,200,0)
factor = 361 /240
display.set_pen(80,80,80)
horiz(0,180,239)    
display.update()
display.set_pen(0,0,230)
cycle = 2
for x in range(0,240):
    y = int ((math.sin(math.radians(cycle * x * factor)))* -50) + 180
    display.pixel(x,y)
    display.update()
display.text("Sine", 10, 190, 200, 2)
display.update()

display.set_pen(80,80,80)
horiz(0,60,239)    
display.update()
display.set_pen(200,200,0)
for x in range(0,240):
    y = int((math.cos(math.radians(cycle * x * factor)))* -50) + 60
    display.pixel(x,y)
display.text("Cosine", 90, 70, 200, 2)
display.update()
utime.sleep(3)

title("Text on a path",0,0,200)
# Text on a downward slant
display.set_pen(0,240,0)
msg =" ++ Hackspace + Pico ++"
b = bytes(msg, 'utf-8')
for i in range(len(b)):
    c = b[i]
    display.character(c, i*10,i*5 +110,2)
    display.update()

# Text on a Sin wave
factor = 361 /240
display.set_pen(255,0,0)
for i in range(len(b)):
    y = int ((math.sin(math.radians(i*10 * factor)))* -50) + 50
    c = b[i]
    display.character(c, i*10,y +10,2)
    display.update()
utime.sleep(4)

title("Scrolling text on a Sine Curve",0,0,200)
# Scrolling on a Sine curve
# Modified from a method by Tony DiCola for a SSD1306
msg = 'Scrolling text on a sine curve using a pico explorer!'
f_width  = 13   # Font width in pixels
f_height = 10   # Font Height in pixels
amp = 160   # Amplitude of sin wave
freq = 1    # Screen cycles (360 degrees)
   
pos = width  # X position of the first character in the msg.
msg_len_px = len(msg) * f_width  # Pixel width of the msg.
# Extra wide lookup table - calculate once to speed things up
y_table = [0] * (width+f_width) # 1 character extra
for i in range(len(y_table)):
    p = i / (width-1)  # Compute current  position along
    # lookup table in 0 to 1 range.
    # Get y co-ordinate from table
    y_table[i] = int(((amp/2.0) * math.sin(2.0*math.pi*freq*p)) + (amp/2.0))
    
# Scrolling loop:
blk()
running = True
while running:
    # Clear scroll area
    display.set_pen(0,0,0) 
    display.rectangle(0, 0, 250, 200)
    display.set_pen(200,200,0)
    # Start again if msg finished
    pos -= 1
    if pos <= -msg_len_px:
        pos = width
    # Go through each character in the msg.

    for i in range(len(msg)):
        char = msg[i]
        char_x = pos + (i * f_width)  # Character's X position on the screen.
        if -f_width <= char_x < width:
            # If haracter is visible, draw it.
            display.text(char, char_x + 5, y_table[char_x+f_width]+20,2)
    display.set_pen(100,100,100)
    display.text("Press button Y to halt", 5, 215, 230, 2)
    display.update()
    if display.is_pressed(3): # Y button is pressed ?
        running = False
    utime.sleep(0.01)
    
# User controlled Graphics
blk()
title("Button control of graphics",0,0,200)
display.set_pen(255,255,255)
display.text("Press A and B buttons", 10, 0, 230, 2)
display.set_pen(200,200,200)
display.text("Press Y button to halt", 5, 15, 230, 2)
display.set_pen(0,100,0)
box(0,43,112,157)
pot = 50
running = True
while running:    
    if display.is_pressed(0): # A pressed
        pot = pot + 1
        if pot > 100: pot = 100
    if display.is_pressed(1): # B pressed
        pot = pot - 1
        if pot < 0 : pot = 0    
    utime.sleep(0.02)    
    # Calculate bargraph colour
    if pot < 50: # Blue -> Cyan -> Green
        r = 0
        g = int(pot * 2)
        b = int(100 - pot * 2)   
    else:       # Green -> Yellow -> Red
        r = int((pot-50)*2)
        g = int(100 - ((pot-50)*2))
        b = 0
    display.update()
    display.set_pen(0,0,0)
    display.circle(56,100,50)
    # Brightness - black to bright grey
    br = 2 * pot
    display.set_pen(br,br,br) 
    display.circle(180,75,30)
    # Circle size
    display.set_pen(70,70,70)
    if (pot > 0):
        display.circle(56,100,int(pot/2))
    # Bar graph & value
    percent = pot
    showgraph(percent,0,40,100)
    # Colour shift: Blue to Green to Red
    display.set_pen(int(r*2.55),int(g*2.55),int(b*2.55))
    display.rectangle(150,120,60,60)
    # Finished?
    if display.is_pressed(3): # Y button is pressed ?
        running = False
      
# Tidy up
blk()
display.set_pen(200,0,0)
display.text("All Done!", 55, 40, 200, 3)
show(2)
blk()
